def ask_until_y_or_n(question):
    var = ''
    while not var in ['Y', 'y', 'N', 'n', 0, 1]: 
        var = input(question + ' (Y/N) ')
    return var in ['y', 'Y', 1]

def delete_duplicate(list):
    seen = set()
    seen_add = seen.add
    return [x for x in list if not (x in seen or seen_add(x))]

def delete_non_numeric(string):
    valid = ''
    for i in range(len(string)):
        if string[i].isnumeric() or string[i] == '.':
            valid += string[i]
    return valid

def str_at_least_n(i, n):
    nb = str(i)
    while len(nb) < n:
        nb = "0" + nb
    return nb


from requests_html import HTMLSession
import json

import MangaDexPy

from .Headers.functions_web import does_page_exists
from .Headers.errors import ConnexionError
from .Headers.functions import delete_non_numeric, delete_duplicate

try:
    with open("mangas_dl/language_codes.json") as file:
        LANGUAGE_CODES = json.load(file)
except:
    raise FileNotFoundError("The file websites_used.json has not been found.\nPlease make sure it exists before lauching mangas-dl.")

def manganelo_pre_download(url, choosen_language):
    session = HTMLSession()
    r = session.get(url)
    manga_name = r.html.find('h1')[0].html[4:-5]

    url_chapter1 = url.split('/')
    url_chapter1[-2] = 'chapter'
    url_chapter1.append('chapter-')
    url_chapter1 = '/'.join(url_chapter1) + '1'
    
    if not does_page_exists(url_chapter1):
        raise ConnexionError(url_chapter1)

    r = session.get(url_chapter1)

    pre_chapters = r.html.find('.navi-change-chapter')[0].html.split(" ")

    chapters = []
    chapters_name = []
    for i in range(len(pre_chapters)-2, -1, -1):
        if 'Chapter' in pre_chapters[i]:
            test = delete_non_numeric(pre_chapters[i+1].split('\n')[0])
            if not test in ['.', '']:
                chapters.append(test)

    chapters = delete_duplicate(chapters)
    for i in range(len(chapters)):
        chapters_name.append(chapters[i])
        if ':' in chapters_name[-1]:
            chapters_name[-1] = chapters_name[-1][:-1]

        if not '.' in chapters_name[-1]:
            chapters_name[-1] += '.0'
        while len(chapters_name[-1]) < 5:
            chapters_name[-1] = '0' + chapters_name[-1]

    return chapters, chapters_name, manga_name

def mangadex_pre_download(url, choosen_language = -1):
    mangadex_client = MangaDexPy.MangaDex()
    manga = mangadex_client.get_manga(url.split('/')[-1])
    all_languages_chapters = manga.get_chapters()

    available_languages = delete_duplicate([chp.language for chp in all_languages_chapters])

    if len(available_languages) == 1:
        choosen_language = 0
    elif choosen_language == -1:
        print(len(available_languages), "languages have been found.")
        for i in range(len(available_languages)):
            try:
                print(i, "->", LANGUAGE_CODES[available_languages[i]])
            except:
                print(i, "->", available_languages[i])
        while choosen_language == -1:
            choosen_language = available_languages[int(input("Choose a language (number) : "))]

    chapters = [chp for chp in all_languages_chapters if chp.language == choosen_language]
    chapters_name = [chp.chapter for chp in chapters]

    i = 0
    while i < len(chapters_name):
        j = i + 1

        if chapters_name[i] == None:
            chapters_name.pop(i)
            chapters.pop(i)
            i -= 1
        else:
            while j < len(chapters_name):
                if chapters_name[i] == chapters_name[j]:
                    chapters_name.pop(j)
                    chapters.pop(j)
                    j -= 1
                j += 1
        i += 1

    chapters = [chp for _, chp in sorted(zip(list(map(float, chapters_name)), chapters), key=lambda pair : pair[0])]
    chapters_name = list(map(str, list(sorted(map(float, chapters_name)))))

    return chapters, chapters_name, manga.title["en"]


import requests
from requests_html import HTML, HTMLSession

def does_page_exists(url):
    session = HTMLSession()

    try:
        r = session.get(url)
        return r.status_code == 200
    except:
        return False

def download_image(url, download_path, name, referer = ""):
    headers = {
        "User-Agent" : "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36 OPR/73.0.3856.434",
        "referer" : referer
        }

    image = requests.get(url, stream=True, headers=headers).content
    f = open(download_path + name, 'wb+')
    f.write(image)
    f.close()


import os
from shutil import rmtree
import sys
from requests_html import HTMLSession
from PIL import Image

from MangaDexPy import downloader

from .Headers.download_and_convert import convert_to_pdf, download_and_convert_to_pdf
from .Headers.functions import str_at_least_n
from .Headers.functions_os import enable_print, disable_print

def download_manganelo_tv(url, chapters_asked, download_path, manga_name):
    if not os.path.exists(download_path + manga_name + os.path.sep):
        os.makedirs(download_path + manga_name + os.path.sep)
    download_path = download_path + manga_name + os.path.sep
    session = HTMLSession()

    url_chapter = url.split('/')
    url_chapter.append('chapter-')
    url_chapter[-3] = 'chapter'
    url_chapter = '/'.join(url_chapter)

    for chp_nb in chapters_asked:
        sys.stdout.write('\033[K')
        print('Loading chapter ', chp_nb[0], end='\r')

        if not os.path.exists(download_path + 'temp' + os.path.sep):
            os.makedirs(download_path + 'temp' + os.path.sep)
        
        r = session.get(url_chapter + chp_nb[0])
        images = r.html.find('.img-loading')
        src = [images[i].attrs['data-src'] for i in range(len(images))]

        download_and_convert_to_pdf(src, download_path, chp_nb)

def download_manganelo_com(url, chapters_asked, download_path, manga_name):
    if not os.path.exists(download_path + manga_name + os.path.sep):
        os.makedirs(download_path + manga_name + os.path.sep)
    download_path = download_path + manga_name + os.path.sep
    session = HTMLSession()

    url_chapter = url.split('/')
    url_chapter.append('chapter_')
    url_chapter[-3] = 'chapter'
    url_chapter = '/'.join(url_chapter)

    for chp_nb in chapters_asked:
        sys.stdout.write('\033[K')
        print('Loading chapter ', chp_nb[0], end='\r')

        if not os.path.exists(download_path + 'temp' + os.path.sep):
            os.makedirs(download_path + 'temp' + os.path.sep)
        
        r = session.get(url_chapter + chp_nb[0])
        div = r.html.find('.container-chapter-reader')[0].html.split("=")
        
        src = []
        for i in range(len(div)):
            if 'src' in div[i] and 'https://' in div[i+1]:
                src.append(div[i+1].split('"')[1])

        download_and_convert_to_pdf(src, download_path, chp_nb, referer=url_chapter)

def download_mangadex_org(url, chapters_asked, download_path, manga_name):
    if not os.path.exists(download_path + manga_name + os.path.sep):
        os.makedirs(download_path + manga_name + os.path.sep)
    download_path = download_path + manga_name + os.path.sep

    for chp_src, chp_nb in chapters_asked:
        sys.stdout.write('\033[K')
        print('Loading chapter ', chp_nb, end='\r')

        if not os.path.exists(download_path + 'temp' + os.path.sep):
            os.makedirs(download_path + 'temp' + os.path.sep)

        disable_print()
        downloader.dl_chapter(chp_src, download_path + 'temp' + os.path.sep)
        enable_print()

        extensions = [code.split('.')[-1] for code in chp_src.pages]

        img_list = []
        for i in range(len(chp_src.pages)):
            img_list.append(Image.open(download_path + 'temp' + os.path.sep + str_at_least_n(i + 1, len(str(len(chp_src.pages)))) + '.' + extensions[i]).convert('RGB'))

        convert_to_pdf(img_list, download_path + 'chp_' + chp_nb + '.pdf')

        rmtree(download_path + 'temp' + os.path.sep)


class ConnexionError(Exception):
    def __init__(self, url):
        self.message = "Unnable to connect to this url : " + url

class UnknownWebsiteError(Exception):
    def __init__(self, sitename):
        self.message = sitename + "isn\'t known nowadays."

class InputError(Exception):
    def __init__(self, message = ""):
        self.message = "The format of the input is incorrect. " + message

class OSError(Exception):
    def __init__(self, message):
        self.message = message


import os
from shutil import rmtree
from MangaDexPy import downloader
from PIL import Image

from .functions_web import download_image

def convert_to_pdf(img_list, download_path):
    if len(img_list) > 0:
            image = img_list.pop(0)
            image.save(download_path, save_all=True, append_images=img_list)

def download_and_convert_to_pdf(srcs, download_path, chp_nb, referer=''):
    img_list = []
    for j in range(len(srcs)):
        ext = srcs[j].split('.')[-1]

        download_image(srcs[j], download_path + 'temp' + os.path.sep, 'chp_' + chp_nb[1] + '_' + str(j) + '.' + ext, referer=referer + chp_nb[0])
        img_list.append(Image.open(download_path + 'temp' + os.path.sep + 'chp_' + chp_nb[1] + '_' + str(j) + '.' + ext).convert('RGB'))

    convert_to_pdf(img_list, download_path + 'chp_' + chp_nb[1] + '.pdf')

    rmtree(download_path + 'temp' + os.path.sep)


import os
import sys, getopt
import json

from .mangas_dl_class import Mangas_dl

from .Headers.errors import ConnexionError
from .Headers.functions_formatting import str_to_chapters
from .Headers.functions_os import is_path_exists_or_creatable

__version__ = "0.2.3"

ARGS = sys.argv[1:]

sys.path.append(os.path.dirname(os.path.realpath(__file__)))

def main_one_line(argv):
    try:
        opts, args = getopt.getopt(argv, "hvl:c:p:", ["help", "version", "language", "chapters", "path"])
    except getopt.GetoptError:
        print("#TODO")
        sys.exit(1)

    language = "en"
    chapters_asked = ""
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            f = open("README.md", "r")
            print(f.read())
            f.close()
            sys.exit(0)
        elif opt in ("-v", "--version"):
            print("mangas-dl", __version__)
            sys.exit(0)
        elif opt in ("-l", "--language"):
            language = arg
        elif opt in ("-c", "--chapters"):
            chapters_asked = arg
        elif opt in ("-p", "--path"):
            if arg == "%":
                try:
                    f = open("mangas_dl/settings.json", "r")
                    settings = json.load(f)
                    f.close()
                    download_path = settings["remembered_path"]
                except:
                    raise OSError("There is no remebered path. Save one before using this command.")
            elif is_path_exists_or_creatable(arg):
                download_path = arg
            else:
                print("The given path does not exist and is not creatable. Please try again.")
                sys.exit(2)
    
    if len(args) != 1:
        print("Please enter only one url at time.")
        sys.exit(3)
    elif not "download_path" in locals():
        print("Please enter a path where scans will be saved.")
        sys.exit(2)

    url = args[0]

    mangas_dl = Mangas_dl(url)
    if not mangas_dl.test_connexion():
        raise ConnexionError(url)

    mangas_dl.pre_download(choosen_language = language)
    mangas_dl.chapters_asked = str_to_chapters(mangas_dl.chapters, mangas_dl.chapters_name, chapters_asked)
    mangas_dl.download_path = download_path.rstrip(os.path.sep) + os.path.sep

    return mangas_dl.download_chapters()

def main_interactive(url):
    mangas_dl = Mangas_dl(url)
    if not mangas_dl.test_connexion():
        raise ConnexionError(url)

    mangas_dl.pre_download()
    mangas_dl.ask_chapters_to_download()
    mangas_dl.ask_path()

    return mangas_dl.download_chapters()

def main():
    if len(ARGS) == 0:
        print(main_interactive(input("Enter the main page of the manga you want to download : ")))
    else:
        print(main_one_line(ARGS))

if __name__ == "__main__":
    main()